"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const Reference_1 = require("../Reference");
const TypeLink_1 = require("../Components/TypeLink");
const ArrayTools_1 = require("./ArrayTools");
const TypeAliasHint_1 = require("../Components/TypeAliasHint");
const ReferenceNodeKind_1 = require("../Reference/ReferenceNodeKind");
exports.isOptionalType = (def) => {
    if (!def) {
        return true;
    }
    if (def.type === 'union') {
        if (def.types.length === 2) {
            const undefIndex = def.types.findIndex(type => type.type === 'intrinsic' && type.name === 'undefined');
            if (undefIndex !== -1) {
                return true;
            }
        }
    }
    return false;
};
exports.buildType = (def, ignoreUndefined = false) => {
    if (!def) {
        return 'void';
    }
    switch (def.type) {
        case 'union': {
            if (def.types.length === 2) {
                const undefIndex = def.types.findIndex(type => type.type === 'intrinsic' && type.name === 'undefined');
                if (undefIndex !== -1) {
                    const defIndex = +!undefIndex; // 0 => 1, 1 => 0 to find the type that isn't undefined
                    if (ignoreUndefined) {
                        return exports.buildType(def.types[defIndex], true);
                    }
                    return React.createElement(React.Fragment, null,
                        "?",
                        exports.buildType(def.types[defIndex]));
                }
            }
            return (React.createElement(React.Fragment, null, def.types.map((type, idx) => (React.createElement(React.Fragment, { key: idx },
                idx !== 0 ? ' | ' : '',
                exports.buildType(type))))));
        }
        case 'array': {
            return React.createElement(React.Fragment, null,
                exports.buildType(def.elementType),
                "[]");
        }
        case 'stringLiteral': {
            return React.createElement(React.Fragment, null,
                "\"",
                def.value,
                "\"");
        }
        case 'reflection': {
            // TODO
            return '';
        }
        default: {
            if (def.type === 'reference' && def.id) {
                const referencedType = ArrayTools_1.findByMember(Reference_1.default.children, 'id', def.id);
                if (referencedType && referencedType.kind === ReferenceNodeKind_1.ReferenceNodeKind.TypeAlias) {
                    return React.createElement(TypeAliasHint_1.default, { name: referencedType.name, type: referencedType.type });
                }
            }
            return (React.createElement(React.Fragment, null,
                def.type === 'reference' && def.id ? React.createElement(TypeLink_1.default, { name: def.name }, def.name) : def.name,
                def.type === 'reference' && def.typeArguments ? (React.createElement(React.Fragment, null,
                    "<",
                    def.typeArguments.map((param, idx) => (React.createElement(React.Fragment, { key: idx },
                        idx !== 0 ? ', ' : '',
                        exports.buildType(param)))),
                    ">")) : null));
        }
    }
};
exports.getTag = (node, name) => {
    name = name.toLowerCase();
    if (!node || !node.comment || !node.comment.tags) {
        return null;
    }
    const foundTag = node.comment.tags.find(tag => tag.tag === name);
    return foundTag ? foundTag.text : null;
};
exports.hasTag = (node, name) => exports.getTag(node, name) !== null;
exports.isStringLiteral = (def) => {
    if (!def) {
        return false;
    }
    switch (def.type) {
        case 'stringLiteral':
            return true;
        case 'union':
            return def.types.every(exports.isStringLiteral);
        default:
            return false;
    }
};
exports.getPageType = (node) => {
    switch (node.kind) {
        case ReferenceNodeKind_1.ReferenceNodeKind.Class:
            return 'classes';
        case ReferenceNodeKind_1.ReferenceNodeKind.Enum:
            return 'enums';
        case ReferenceNodeKind_1.ReferenceNodeKind.Interface:
            return 'interfaces';
        default:
            return 'unknown';
    }
};
//# sourceMappingURL=CodeBuilders.js.map