"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const commonmark = require("commonmark");
const ReactRenderer = require("commonmark-react-renderer");
const react_router_hash_link_1 = require("react-router-hash-link");
const light_1 = require("react-syntax-highlighter/light");
// tslint:disable-next-line:match-default-export-name
const darcula_1 = require("react-syntax-highlighter/styles/hljs/darcula");
const CodeBuilders_1 = require("./CodeBuilders");
const ArrayTools_1 = require("./ArrayTools");
const Reference_1 = require("../Reference");
function parseMarkdown(source) {
    const parser = new commonmark.Parser();
    const intermediate = parser.parse(source);
    const walker = intermediate.walker();
    let event;
    let node;
    // tslint:disable-next-line:no-conditional-assignment
    while (event = walker.next()) {
        node = event.node;
        // transform linked type names
        if (node.type === 'text') {
            const re = /{@((\w+)(?:#(\w+))?)}/g;
            let match;
            // tslint:disable-next-line:no-conditional-assignment
            while (node.literal && (match = re.exec(node.literal))) {
                const [fullMatch, fullSymbolName, symbolName, memberName] = match;
                const entry = ArrayTools_1.findByMember(Reference_1.default.children, 'name', symbolName);
                if (!entry) {
                    continue;
                }
                const pageType = CodeBuilders_1.getPageType(entry);
                if (!pageType) {
                    continue;
                }
                if (match.index > 0) {
                    const beforeText = new commonmark.Node('text');
                    beforeText.literal = node.literal.substr(0, match.index);
                    node.insertBefore(beforeText);
                }
                const link = new commonmark.Node('link');
                link.destination = `/reference/${pageType}/${symbolName}`;
                if (memberName) {
                    link.destination += `#symbol__${memberName}`;
                }
                const linkText = new commonmark.Node('text');
                linkText.literal = fullSymbolName;
                link.appendChild(linkText);
                node.insertBefore(link);
                node.literal = node.literal.substr(match.index + fullMatch.length);
                if (!node.literal) {
                    node.unlink();
                    break;
                }
            }
        }
    }
    // noinspection JSUnusedGlobalSymbols
    const renderer = new ReactRenderer({
        renderers: {
            // tslint:disable-next-line:no-any
            link: function MdLink(mdProps) {
                const props = {
                    key: mdProps.nodeKey,
                    className: mdProps.className
                };
                if (mdProps.href.startsWith('/')) {
                    return (React.createElement(react_router_hash_link_1.HashLink, Object.assign({}, props, { to: mdProps.href }), mdProps.children));
                }
                else {
                    return (React.createElement("a", { href: mdProps.href }, mdProps.children));
                }
            },
            // tslint:disable-next-line:no-any
            code_block: function MdCodeBlock(mdProps) {
                return (React.createElement(light_1.default, { key: mdProps.nodeKey, language: mdProps.language, style: darcula_1.default }, mdProps.literal));
            }
        }
    });
    return renderer.render(intermediate);
}
exports.default = parseMarkdown;
//# sourceMappingURL=MarkdownParser.js.map