"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const Generator_1 = require("./Generator");
const path = require("path");
const resolveHome = require("untildify");
const webpack = require("webpack");
const tmp = require("tmp");
const fs = require("fs-extra");
const WebpackError_1 = require("../Errors/WebpackError");
const WebpackBuildError_1 = require("../Errors/WebpackBuildError");
const ArrayTools_1 = require("../../Common/Tools/ArrayTools");
const ReferenceNodeKind_1 = require("../../Common/Reference/ReferenceNodeKind");
class HTMLGenerator extends Generator_1.default {
    generate(data) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const outDir = path.resolve(this._config.baseDir, resolveHome(this._config.outputDir));
            yield fs.emptyDir(outDir);
            return new Promise((resolve, reject) => {
                tmp.dir({ unsafeCleanup: true }, (err, tmpDir, cleanup) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                    if (err) {
                        reject(err);
                        return;
                    }
                    try {
                        yield this._buildWebpack(data, tmpDir);
                    }
                    catch (e) {
                        reject(e);
                        return;
                    }
                    yield fs.copy(path.join(tmpDir, 'static'), path.join(outDir, 'static'));
                    const { default: render } = require(path.resolve(tmpDir, 'generator.js'));
                    try {
                        yield Promise.all([
                            ['/', this._config.indexTitle, fs.readFile(this._config.indexFile, 'utf-8')],
                            ...[].concat(...((this._config.configDir && this._config.categories) ? this._config.categories.map(cat => cat.articles.map(art => [
                                `/docs/${cat.name}/${art.name}`, art.title, fs.readFile(path.join(this._config.configDir, art.file), 'utf-8')
                            ])) : [])),
                            ...ArrayTools_1.filterByMember(data.children, 'kind', ReferenceNodeKind_1.ReferenceNodeKind.Class).map(value => `/reference/classes/${value.name}`),
                            ...ArrayTools_1.filterByMember(data.children, 'kind', ReferenceNodeKind_1.ReferenceNodeKind.Interface).map(value => `/reference/interfaces/${value.name}`),
                            ...ArrayTools_1.filterByMember(data.children, 'kind', ReferenceNodeKind_1.ReferenceNodeKind.Enum).map(value => `/reference/enums/${value.name}`)
                        ].map((entry) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                            if (Array.isArray(entry)) {
                                const [resourcePath, title, contentPromise] = entry;
                                return this._renderToFile(render, resourcePath, outDir, {
                                    content: yield contentPromise,
                                    title
                                });
                            }
                            return this._renderToFile(render, entry, outDir);
                        })));
                    }
                    catch (e) {
                        reject(e);
                        return;
                    }
                    finally {
                        cleanup();
                    }
                    resolve();
                }));
            });
        });
    }
    _renderToFile(render, resourcePath, outDir, content) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let relativeOutFile = resourcePath;
            if (resourcePath.endsWith('/')) {
                relativeOutFile += 'index.html';
            }
            else if (this._config.routerMode === 'subDirectories') {
                relativeOutFile += '/index.html';
            }
            else {
                relativeOutFile += '.html';
            }
            const outFile = path.join(outDir, relativeOutFile);
            yield fs.mkdirp(path.dirname(outFile));
            const str = render(resourcePath, content);
            yield fs.writeFile(outFile, str);
        });
    }
    _buildWebpack(data, outputDirectory) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                process.chdir(path.join(__dirname, '../../..'));
                // tslint:disable-next-line:no-var-requires
                const webpackConfig = require('../../../config/webpack.config.html')(outputDirectory);
                const webpackCompiler = webpack(webpackConfig);
                const _a = this._config, { webpackProgressCallback } = _a, configWithoutCallback = tslib_1.__rest(_a, ["webpackProgressCallback"]);
                if (webpackProgressCallback) {
                    (new webpack.ProgressPlugin(webpackProgressCallback)).apply(webpackCompiler);
                }
                (new webpack.DefinePlugin({
                    __DOCTS_REFERENCE: JSON.stringify(data),
                    __DOCTS_CONFIG: JSON.stringify(configWithoutCallback)
                })).apply(webpackCompiler);
                webpackCompiler.run((err, stats) => {
                    if (err) {
                        reject(new WebpackError_1.default(err));
                    }
                    else if (stats.hasErrors()) {
                        reject(new WebpackBuildError_1.default(stats));
                    }
                    else {
                        resolve();
                    }
                });
            });
        });
    }
}
exports.default = HTMLGenerator;
//# sourceMappingURL=HTMLGenerator.js.map