"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const Generator_1 = require("./Generator");
const path = require("path");
const resolveHome = require("untildify");
const webpack = require("webpack");
const WebpackError_1 = require("../Errors/WebpackError");
const WebpackBuildError_1 = require("../Errors/WebpackBuildError");
class SPAGenerator extends Generator_1.default {
    generate(data) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                process.chdir(path.join(__dirname, '../../..'));
                // tslint:disable-next-line:no-var-requires
                const webpackConfig = require('../../../config/webpack.config.spa');
                webpackConfig.output.path = path.resolve(this._config.baseDir, resolveHome(this._config.outputDir));
                const webpackCompiler = webpack(webpackConfig);
                const _a = this._config, { webpackProgressCallback } = _a, configWithoutCallback = tslib_1.__rest(_a, ["webpackProgressCallback"]);
                if (webpackProgressCallback) {
                    (new webpack.ProgressPlugin(webpackProgressCallback)).apply(webpackCompiler);
                }
                (new webpack.DefinePlugin({
                    __DOCTS_REFERENCE: JSON.stringify(data),
                    __DOCTS_CONFIG: JSON.stringify(configWithoutCallback)
                })).apply(webpackCompiler);
                webpackCompiler.run((err, stats) => {
                    if (err) {
                        reject(new WebpackError_1.default(err));
                    }
                    else if (stats.hasErrors()) {
                        reject(new WebpackBuildError_1.default(stats));
                    }
                    else {
                        resolve();
                    }
                });
            });
        });
    }
}
exports.default = SPAGenerator;
//# sourceMappingURL=SPAGenerator.js.map